#    Lunar Rover Expedition - A fun adventure game.
#    Copyright (C) 2013 Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importion of modules
import pygame
pygame.init()

from code.other.helpers import load_image

class Animated_Sprite( pygame.sprite.Sprite ):
	"""
	A class that can be used directly, or inherited from, that supports simple animation.
	"""


	def __init__( self, images_file_names, animation_intervals ):
		"""
		Parameters
			images_file_names	( string, string, ... ) or [ string, string, ... ]
				The file names of the images to use for the animation.
			animation_intervals	( int, int, ... )
				Defines the speed of the animation.  A speed of 5 would update the animated sprite every 5 frames.
				One can pass multiple intervals.  This means that the animation either increases or decreases in speed.

				The animation interval is changed everytime after a new image has been chosen.
		"""
		pygame.sprite.Sprite.__init__( self )

		self._images_file_names = images_file_names
		self._animation_intervals = animation_intervals
		self._shown_image = 0
		self._chosen_animation_interval = 0
		self._current_animation_interval = animation_intervals[self._chosen_animation_interval]
		self._animation_interval_timer = 0

		# Load and store the images in a list.
		self._animation_images = []
		for image_file_name in images_file_names:
			self._animation_images.append( load_image( image_file_name ) )

		# Set the first image of that list to be the start image of the animation.
		self.image = self._animation_images[self._shown_image]


	def update_animation( self ):
		"""
		Sets the current image to be the next image of the list.
		If the end of the animation images list has been reached, start from the beginning again.
		If the first image is chosen again, True is returned.  This has been implemented in order to kill
		a sprite after the animation has finished the first time.
		"""

		# Check if the next image should be shown (animation speed).
		if self._animation_interval_timer == self._current_animation_interval:
			# Change the image.
			if len( self._animation_images ) > self._shown_image + 1: # + 1:  A list is indexed at 0.
				# Grab the next image.
				self._shown_image += 1
				self.image = self._animation_images[self._shown_image]

				# The animation cycle has not yet finished
				animation_restarted = False
				
			else:
				# Show the first image again.
				self._shown_image = 0
				self.image = self._animation_images[self._shown_image]

				# Let the calling code know that the animation cycle has re-started from the beginning.
				animation_restarted = True

			# Select the next animation interval
			self.select_next_animation_interval()
			self._animation_interval_timer = 0

			return animation_restarted
			
		else:
			self._animation_interval_timer += 1


	def select_next_animation_interval( self ):
		"""
		Selects the next animation interval from the animation interval list.
		If the last one has been chonsen, then the next will be the first again.
		"""

		if len( self._animation_intervals ) > self._chosen_animation_interval + 1:
			# Grab the next animation interval.
			self._chosen_animation_interval += 1
			self._current_animation_interval = self._animation_intervals[self._chosen_animation_interval]
		else:
			self._chosen_animation_interval = 0
