#    Lunar Rover Expedition - A fun adventure game.
#    Copyright (C) 2013 Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importion of modules
import math
import copy
import pygame
pygame.init()

from code.classes.Enemy import Enemy
from code.classes.Shot import Shot


class Moving_Enemy( Enemy ):
	"""
	This class represents an enemy that can move and shoot.
	"""

	def __init__( self, image_file_names_enemy, animation_intervals_enemy, health_points, projectile_damage, name, left_top_position, shooting_interval, moving_speed, shot_data, min_distance, detection_distance ):
		"""
		Parameters
		"""
		super( Moving_Enemy, self ).__init__( image_file_names_enemy, animation_intervals_enemy, health_points, projectile_damage, name, left_top_position, detection_distance )

		# Store some of the parameters that have been passed.
		self._shot_data = copy.deepcopy( shot_data )

		self._shooting_interval = shooting_interval
		self._shooting_interval_counter = 0

		# I save the shots in a sprite group.
		self._fired_shots = pygame.sprite.Group()

		self._lunar_rover_within_range = True

		self._direction_angle = 0 # Will be set throughout the game.

		self._moving_speed = moving_speed
		self._distance = 0

		self._min_distance = min_distance
		self._orig_distance = self._min_distance

		self._detection_distance = detection_distance
		self._sleeping = True # True means == Do not fire; do not move.


	def update( self, lunar_rover_xy_velocity ):
		"""
		Parameters
			lunar_rover_xy_velocity
				Defines where the enemy should be moved to when the player moves with his lunar rover.
		"""
		
		super( Moving_Enemy, self ).update( lunar_rover_xy_velocity )

		# Is the player within range?
		if not self._sleep:
			if self.is_fire_new_shot():
				# Fire a shot.	
				self._shot_data["center_start_pos"] = self.rect.center
				self._fired_shots.add( Shot( **self._shot_data ) )

		# Update any shots that have been fired.
		self._fired_shots.update()

		# Make sure the shots stay in the same position and do not "alter" they angle.
		for s in self._fired_shots:
			s.enemy_keep_shot_straight( lunar_rover_xy_velocity )

		# Calculate the x and y velocities, so the enemy rover can move towards the player or away from him.
		velocity_x, velocity_y = self.calculate_xy_velocity()

		
		# Move the moving enemy towards the player.
		# Only move it the distance is more or equal to 100 px.

		# Only move if the enemy is not sleeping
		if not self._sleep:
			if self._distance > self._min_distance:
				self.rect.move_ip( velocity_x, velocity_y )
				self._min_distance = self._orig_distance
			else:
				self.rect.move_ip( velocity_x * -1, velocity_y * -1 )

	def is_fire_new_shot( self ):
		"""
		Returns true if a new shot can be fired.
		"""

		if self._shooting_interval_counter >= self._shooting_interval:
			self._shooting_interval_counter = 0
			return True
		else:
			self._shooting_interval_counter += 1
			return False


	def set_shot_angle( self, angle ):
		"""
		Sets the shooting angle (in degrees).

		Parameters
			angle			int
				The angle to shoot at.
		"""

		self._shot_data["angle"] = angle
		self.set_direction_angle( angle )


	def set_direction_angle( self, angle ):
		"""
		Sets the angle of where the enemy should go to.  Usually towards the player.

		Parameters
			angle			int
				The angle of where the kamikaze enemy should move to.
		"""

		self._direction_angle = angle


	def calculate_xy_velocity( self ):
		"""
		Calculates the velocity along the x and y axes.
		"""

		velocity_x = self._moving_speed * math.sin( math.radians( self._direction_angle ) ) * -1
		velocity_y = self._moving_speed * math.cos( math.radians( self._direction_angle ) ) * -1

		return velocity_x, velocity_y
