#    Lunar Rover Expedition - A fun adventure game.
#    Copyright (C) 2013 Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importion of modules
import datetime
import collections
import math
import os.path
import os
import json

import pygame
pygame.init()

from constants import IMAGES_DIR_PATH, SOUNDS_DIR_PATH, MUSIC_DIR_PATH, HIGHSCORES_DIR_PATH
from code.classes.Transition import Transition


def load_image( file_name ):
	"""
	Loads an image and returns a pygame.Surface object.

	Parameters
		file_name	string
			The name of the image to load.

	Return value
		image		pygame.Surface
			The loaded image.
	"""

	try:
		image = pygame.image.load( os.path.join( IMAGES_DIR_PATH, file_name ) )
		image = image.convert_alpha()
		return image
	except:
		raise pygame.error( "The image '%s' could not be loaded." % file_name )


def load_sound( file_name ):
	"""
	Loads a sound and returns a pygame.mixer.Sound object.
	The sound object should be in .ogg or .WAV/.wav.

	Parameters
		file_name	string
			The name of the sound to load.

	Return value
		sound		pygame.mixer.Sound
			The loaded sound.
	"""

	try:
		sound = pygame.mixer.Sound( os.path.join( SOUNDS_DIR_PATH, file_name ) )
		return sound
	except:
		pygame.error( "The sound '%s' could not be loaded." % file_name )


def load_music( file_name ):
	"""
	Loads a music track and returns a pygame.mixer.Sound object.
	The sound object should be in .ogg or .WAV/.wav.

	Parameters
		file_name	string
			The name of the music track to load.

	Return value
		sound		pygame.mixer.Sound
			The loaded music track.
	"""

	try:
		music_track = pygame.mixer.Sound( os.path.join( SOUNDS_DIR_PATH, file_name ) )
		return music_track
	except:
		pygame.error( "The music track '%s' could not be loaded." % file_name )


def collision_lunar_rover_level_objects( lunar_rover, level_objects ):
	"""
	A function that checks for collisions between the lunar rover and the objects of the level.
	If a collision has happened, something else happens.  For more information, please see the function.

	Parameters
		lunar_rover		Lunar_Rover
			The lunar rover object.
		level_objects	pygame.sprite.Group
			Contains all the objects of a level.  All of them are sprites.  They can either be scenery objects, obstacle objects, samples objects or enemy objects.
	"""

	# Find all sprites that collide with the player.
	
	# Since the players vehicle, the lunar rover, is too big, I need to use its smaller collision rectangle for the collision.  Otherwise, the lunar rover would collide with a level object, without being anywhere near it.
	player_copy = pygame.sprite.Sprite()
	player_copy.rect = lunar_rover._collision_rect
	
	collided_sprites = pygame.sprite.spritecollide( player_copy, level_objects, False )

	# Next, I go through each of the collided sprites and check, to which class the belong.
	# Based onto which class they belong, I either remove them from the game, or do something completely different.
	for col_sprite in collided_sprites:
		if col_sprite.__class__.__name__ == "Sample":
			# The lunar rover / player has found a sample!  Remove that sample.
			col_sprite.kill()
			# A new sample has been collected.  Alter the sample meter.
			lunar_rover._sample_meter.increase_meter( 1 )
			lunar_rover._sound_sample_collected.play()
		elif col_sprite.__class__.__name__ == "Kamikaze_Enemy":
			# The lunar rover has been hit by a kamikaze enemy.
			col_sprite.kill()
			# Inflict damage to the lunar rover.
			lunar_rover._vehicle_health.reduce_health( col_sprite._projectile_damage )


def collision_lunar_rover_shots_and_enemies_shots( lunar_rover, obstacles, shooting_enemies ):
	"""
	A function that checks for collisions with shots, the lunar rover of the player and the obstacles of a level.
	If a collision occurs, some action occurs; that means, that some functions are called that do something.
	"""
	
	# Check for any collision with shots.  First, for the player.
	collided_sprites = pygame.sprite.groupcollide( lunar_rover._shots, obstacles, False, False )
	for key_val in collided_sprites.iteritems():
		# Remove the shot which the player has fired.
		key_val[0].kill()

		# The shot, which the player has fired, does do some damage to the obstacle it has hit.
		for obstacle in key_val[1]:
			obstacle.reduce_health_points( lunar_rover._shot_damage )


	# Now I do the same, but for the enemies.

	# Since the players vehicle, the lunar rover, is too big, I need to use its smaller collision rectangle for the collision.  Otherwise, the lunar rover would collide with a level object, without being anywhere near it.
	lunar_rover_copy = pygame.sprite.Sprite()
	lunar_rover_copy.rect = lunar_rover._collision_rect

	for shooting_enemy in shooting_enemies:
		enemies_fired_shots = pygame.sprite.spritecollide( lunar_rover_copy, shooting_enemy._fired_shots, False )

		# The shots that have collided with the lunar rover, the player's lunar rover, do some damage to it.
		# Maybe they destroy it completely.
		for fired_shot in enemies_fired_shots:
			lunar_rover._vehicle_health.reduce_health( shooting_enemy._projectile_damage )
			# And remove the shot that has been fired.
			fired_shot.kill()
			


def calc_angle_between_obstacles_and_player_lunar_rover( lunar_rover, obstacles ):
	"""
	Calculates the angle of an obstacle and sets it.
	The angle is calculated by taking into account the position of the lunar rover, which the player controls.

	This angle is used for fireing projectiles towards the player, aswell as following him.
	"""

	# I store the center coords of the player's rover.
	lr_center = lunar_rover.rect.center

	# I walk through all obstacles.
	for ob in obstacles:
		# I store the center of the current obstacle.
		ob_center = ob.rect.center

		# I subtract the obstacle center position from the rover center position
		# I get cartesian coordinates.
		c_coords = [ lr_center[0] - ob_center[0], lr_center[1] - ob_center[1] ]

		# Then I calculte the atan(y/x) in radians.
		angle_radians = math.atan2( c_coords[1], c_coords[0] )

		# I then convert this to degree.
		angle_degree = math.degrees( angle_radians )

		# And I get my angle (hopefully).
		# I set the angle of the obstacle to this angle.
		if ob.__class__.__name__ is not "Kamikaze_Enemy":
			ob.set_shot_angle( ( angle_degree + 90 ) * -1 )
		else:
			ob.set_direction_angle( ( angle_degree + 90 ) * -1 )


def calc_distance_moving_enemy_lunar_rover( obstacles, lunar_rover ):

	for o in obstacles:
		o.calculate_distance_to_lunar_rover( lunar_rover.rect.center )


def run_transition( playground, clock, fps, transition_data, optional_bg=None, blit_before=False ):
	"""
	Runs a transition.

	Parameters
		transition_data			dict
			A dict that contains the data to create a transition object.
		playground				pygame.Surface
			A pygame.Surface object.  The transition is drawn onto this.
		clock					pygame.time.Clock
			Defines how often to update the screen in a second.
		fps						int
			The frame-rate.
		optional_bg				pygame.sprite.Sprite
			A sprite that cointains an optional background image to be blitted first.  Usually only used for transitions that fade out.
		blit_before
			Blit the optional background before the main loop is run?
	"""

	transition = Transition( **transition_data )

	if blit_before:
		transition.image.blit( optional_bg.image, optional_bg.rect )
		
	while transition.update():
		eventlist = pygame.event.get()
		if optional_bg is not None:
			#optional_bg.update()
			if not blit_before:
				playground.blit( optional_bg.image, optional_bg.rect )
		playground.blit( transition.image, transition.rect )
		clock.tick( fps )
		pygame.display.flip()

	return transition


def read_highscore( highscore_filename ):
	"""
	This function reads the content of the highscore file and returns the dictionary to the calling class/function.
	Since the highscore file is decoded in JSON format, I am going to make use of the 'json' module.
	"""

	# Open the file for reading.
	file_handle = open( os.path.join( HIGHSCORES_DIR_PATH, highscore_filename ), "r" )

	# Read the contents.  The content is stored in a dict.
	highscore_data = json.load( file_handle )

	# And close the file as it is no longer needed.
	file_handle.close()

	return highscore_data


def write_highscore( highscore_filename, highscore_data ):
	"""
	"""

	new_highscore = {}
	counter = 1
	for item in highscore_data.itervalues():
		new_highscore.update( { str( counter ): item } )
		counter += 1
		

	# Open the file for writing.
	file_handle = open( os.path.join( HIGHSCORES_DIR_PATH, highscore_filename ), "w" )

	# And write the highscores into the file using json.dump
	json.dump( new_highscore, file_handle )	

	# Close the file
	file_handle.close()

	# We are done.


def check_highscores( player_name, player_time, highscore_filename ):
	"""
	This function checks if the player has made it into the highscores.
	If he did, then add his rank to the highscore.
	"""

	# Will be set to 'True' if highscores have to be re-written to the file.
	# This is only the case if the player made it into the highscores :).
	write_score = False

	# Read the highscore.
	highscore = read_highscore( highscore_filename )

	# Now sort the highscore that has just been read.
	sorted_highscore_table = collections.OrderedDict( sorted( highscore.items(), key=lambda t: t[1] ) )

	for rank in sorted_highscore_table.itervalues():
		temp_time = datetime.datetime.strptime( rank["score"], "%H:%M:%S" )

		if temp_time.time() > player_time.time():
			rank["name"] = player_name
			rank["score"] = player_time.strftime( "%H:%M:%S" )
			write_score = True
			break


	if write_score:
		# And write the highscores to the file again, as the player made it into the highscores.
		write_highscore( highscore_filename, sorted_highscore_table )
